<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 08/31/2008 *
 * ===============                              Last change: 08/31/2008 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : surfbar_functions.php                            *
 * -------------------------------------------------------------------- *
 * Short description : Functions for surfbar                            *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Funktionen fuer die Surfbar                      *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * Needs to be in all Files and every File needs "svn propset           *
 * svn:keywords Date Revision" (autoprobset!) at least!!!!!!            *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), '/inc') + 4) . '/security.php';
	require($INC);
}

// -----------------------------------------------------------------------------
//                               Admin functions
// -----------------------------------------------------------------------------
//
// Admin has added an URL with given user id and so on
function SURFBAR_ADMIN_ADD_URL ($url, $limit, $reload) {
	// Do some pre-checks
	if (!IS_ADMIN()) {
		// Not an admin
		DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("Cannot add URL=%s,limit=%s,reload=%s : Not admin.", $url, $limit, $reload));
		return false;
	} elseif (!isUrlValid($url)) {
		// URL invalid
		DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("Cannot add URL=%s,limit=%s,reload=%s : Invalid URL.", $url, $limit, $reload));
		return false;
	} elseif (SURFBAR_LOOKUP_BY_URL($url, '0')) {
		// URL already found in surfbar!
		DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("Cannot add URL=%s,limit=%s,reload=%s : Already added.", $url, $limit, $reload));
		return false;
	} elseif (!SURFBAR_IF_USER_BOOK_MORE_URLS()) {
		// No more allowed!
		DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("Cannot add URL=%s,limit=%s,reload=%s : No more URLs allowed.", $url, $limit, $reload));
		return false;
	} elseif ('' . ($limit + 0) . '' != '' . $limit . '') {
		// Invalid limit entered
		DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("Cannot add URL=%s,limit=%s,reload=%s : Invalid limit entered.", $url, $limit, $reload));
		return false;
	} elseif ('' . ($reload + 0) . '' != '' . $reload . '') {
		// Invalid amount entered
		DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("Cannot add URL=%s,limit=%s,reload=%s : Invalid reload entered.", $url, $limit, $reload));
		return false;
	}

	// Register the new URL
	return SURFBAR_REGISTER_URL($url, '0', 'ACTIVE', 'unlock', array('limit' => $limit, 'reload' => $reload));
}

// Admin unlocked an email so we can migrate the URL
function SURFBAR_ADMIN_MIGRATE_URL ($url, $uid) {
	// Do some pre-checks
	if (!IS_ADMIN()) {
		// Not an admin
		return false;
	} elseif (!isUrlValid($url)) {
		// URL invalid
		return false;
	} elseif (SURFBAR_LOOKUP_BY_URL($url, $uid)) {
		// URL already found in surfbar!
		return false;
	} elseif (!SURFBAR_IF_USER_BOOK_MORE_URLS($uid)) {
		// No more allowed!
		return false;
	}

	// Register the new URL
	return SURFBAR_REGISTER_URL($url, $uid, 'MIGRATED', 'migrate');
}

// Admin function for unlocking URLs
function SURFBAR_ADMIN_UNLOCK_URL_IDS ($IDs) {
	// Is this an admin or invalid array?
	if (!IS_ADMIN()) {
		// Not admin or invalid IDs array
		return false;
	} elseif (!is_array($IDs)) {
		// No array
		return false;
	} elseif (count($IDs) == 0) {
		// Empty array
		return false;
	}

	// Set to true to make AND expression valid if first URL got unlocked
	$done = true;

	// Update the status for all ids
	foreach ($IDs as $id => $dummy) {
		// Test all ids through (ignores failed)
		$done = (($done) && (SURFBAR_CHANGE_STATUS($id, 'PENDING', 'ACTIVE')));
	} // END - if

	// Return total status
	return $done;
}

// Admin function for rejecting URLs
function SURFBAR_ADMIN_REJECT_URL_IDS ($IDs) {
	// Is this an admin or invalid array?
	if (!IS_ADMIN()) {
		// Not admin or invalid IDs array
		return false;
	} elseif (!is_array($IDs)) {
		// No array
		return false;
	} elseif (count($IDs) == 0) {
		// Empty array
		return false;
	}

	// Set to true to make AND expression valid if first URL got unlocked
	$done = true;

	// Update the status for all ids
	foreach ($IDs as $id => $dummy) {
		// Test all ids through (ignores failed)
		$done = (($done) && (SURFBAR_CHANGE_STATUS($id, 'PENDING', 'REJECTED')));
	} // END - if

	// Return total status
	return $done;
}

//
// -----------------------------------------------------------------------------
//                               Member functions
// -----------------------------------------------------------------------------
//
// Member has added an URL
function SURFBAR_MEMBER_ADD_URL ($url, $limit) {
	// Do some pre-checks
	if (!IS_MEMBER()) {
		// Not a member
		return false;
	} elseif ((!isUrlValid($url)) && (!IS_ADMIN())) {
		// URL invalid
		return false;
	} elseif (SURFBAR_LOOKUP_BY_URL($url, getUserId())) {
		// URL already found in surfbar!
		return false;
	} elseif (!SURFBAR_IF_USER_BOOK_MORE_URLS(getUserId())) {
		// No more allowed!
		return false;
	} elseif (''.($limit + 0).'' != ''.$limit.'') {
		// Invalid amount entered
		return false;
	}

	// Register the new URL
	return SURFBAR_REGISTER_URL($url, getUserId(), 'PENDING', 'reg', array('limit' => $limit));
}

// Create list of actions depending on status for the user
function SURFBAR_MEMBER_ACTIONS ($urlId, $status) {
	// Load all actions in an array for given status
	$actionArray = SURFBAR_getModeAction_ARRAY($status);

	// Init HTML code
	$OUT = "<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">
<tr>\n";

	// Calculate width
	$width = round(100 / count($actionArray));

	// "Walk" through all actions and create forms
	foreach ($actionArray as $actionId => $action) {
		// Add form for this action
		$OUT .= sprintf(LOAD_TEMPLATE('member_surfbar_list_form', true),
		$width,
		bigintval($urlId),
		strtolower($action),
		strtoupper($action),
		strtoupper($action)
		);
	} // END - foreach

	// Close table
	$OUT .= "</tr>
</table>\n";

	// Return code
	return $OUT;
}

// Do the member form request
function SURFBAR_MEMBER_DO_FORM ($formData, $URLs) {
	// By default no action is performed
	$performed = false;

	// Is this a member?
	if (!IS_MEMBER()) {
		// No member!
		return false;
	} elseif ((!isset($formData['id'])) || (!isset($formData['action']))) {
		// Important form elements are missing!
		return false;
	} elseif (!isset($URLs[$formData['id']])) {
		// ID not found in cache
		return false;
	} elseif (!SURFBAR_VALIDATE_MEMBER_ACTION_STATUS($formData['action'], $URLs[$formData['id']]['status'])) {
		// Action not allowed for current URL status
		return false;
	}

	// Secure action
	$action = SQL_ESCAPE(htmlentities(strip_tags($formData['action']), ENT_QUOTES));

	// Has it changed?
	if ($action != $formData['action']) {
		// Invalid data in action found
		return false;
	} // END - if

	// Create the function name for selected action
	$functionName = sprintf("SURFBAR_MEMBER_%s_ACTION", strtoupper($action));

	// Is the function there?
	if (function_exists($functionName)) {
		// Add new status
		$URLs[$formData['id']]['new_status'] = $GLOBALS['cache_array']['surfbar']['new_status'];

		// Extract URL data for call-back
		$urlData = array(merge_array($URLs[$formData['id']], array($action => $formData)));

		// Action found so execute it
		$performed = call_user_func_array($functionName, $urlData);
	} else {
		// Log invalid request
		DEBUG_LOG(__FUNCTION__, __LINE__, " action={$formData['action']},id={$formData['id']},function={$functionName}");
		addFatalMessage(__FUNCTION__, __LINE__, "Invalid member action! action=%s,id=%s,function=%s", array($formData['action'], $formData['id'], $functionName));
	}

	// Return status
	return $performed;
}

// Validate if the requested action can be performed on current URL status
function SURFBAR_VALIDATE_MEMBER_ACTION_STATUS ($action, $status) {
	// Search for the requested action/status combination in database
	$result = SQL_QUERY_ESC("SELECT new_status FROM `{!_MYSQL_PREFIX!}_surfbar_actions` WHERE `action`='%s' AND `status`='%s' LIMIT 1",
	array($action, $status), __FUNCTION__, __LINE__);

	// Is the entry there?
	$isValid = (SQL_NUMROWS($result) == 1);

	// Fetch the new status if found
	if ($isValid) {
		// Load new status
		list($GLOBALS['cache_array']['surfbar']['new_status']) = SQL_FETCHROW($result);
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return status
	return $isValid;
}

//
// -----------------------------------------------------------------------------
//                               Member actions
// -----------------------------------------------------------------------------
//
// Retreat a booked URL
function SURFBAR_MEMBER_RETREAT_ACTION ($urlData) {
	// Create the data array for next function call
	$data = array(
	$urlData['id'] => $urlData
	);

	// Simply change the status here
	return SURFBAR_CHANGE_STATUS ($urlData['id'], $urlData['status'], $urlData['new_status'], $data);
}

// Book an URL now (from migration)
function SURFBAR_MEMBER_BOOKNOW_ACTION ($urlData) {
	// Create the data array for next function call
	$data = array(
	$urlData['id'] => $urlData
	);

	// Simply change the status here
	return SURFBAR_CHANGE_STATUS ($urlData['id'], $urlData['status'], $urlData['new_status'], $data);
}

// Show edit form or do the changes
function SURFBAR_MEMBER_EDIT_ACTION ($urlData) {
	// Is the "execute" flag there?
	if (isset($urlData['edit']['execute'])) {
		// Execute the changes
		return SURFBAR_MEMBER_EXECUTE_ACTION('edit', $urlData);
	} // END - if

	// Display form
	return SURFBAR_MEMBER_DISPLAY_ACTION_FORM('edit', $urlData);
}

// Show delete form or do the changes
function SURFBAR_MEMBER_DELETE_ACTION ($urlData) {
	// Is the "execute" flag there?
	if (isset($urlData['delete']['execute'])) {
		// Execute the changes
		return SURFBAR_MEMBER_EXECUTE_ACTION('delete', $urlData);
	} // END - if

	// Display form
	return SURFBAR_MEMBER_DISPLAY_ACTION_FORM('delete', $urlData);
}

// Pause active banner
function SURFBAR_MEMBER_PAUSE_ACTION ($urlData) {
	return SURFBAR_CHANGE_STATUS($urlData['id'], $urlData['status'], $urlData['new_status'], array($urlData['id'] => $urlData));
}

// Unpause stopped banner
function SURFBAR_MEMBER_UNPAUSE_ACTION ($urlData) {
	// Fix missing entry for template
	$urlData['edit'] = $urlData['unpause'];
	$urlData['edit']['url'] = $urlData['url'];
	$urlData['edit']['limit'] = $urlData['views_max'];

	// Return status change
	return SURFBAR_CHANGE_STATUS($urlData['id'], $urlData['status'], $urlData['new_status'], array($urlData['id'] => $urlData));
}

// Resubmit locked URL
function SURFBAR_MEMBER_RESUBMIT_ACTION ($urlData) {
	return SURFBAR_CHANGE_STATUS($urlData['id'], $urlData['status'], $urlData['new_status'], array($urlData['id'] => $urlData));
}

// Display selected "action form"
function SURFBAR_MEMBER_DISPLAY_ACTION_FORM ($action, $urlData) {
	// Translate some data
	$urlData['registered']    = generateDateTime($urlData['registered'], '2');
	$urlData['views_total']   = translateComma($urlData['views_total']);
	$urlData['views_max']     = translateComma($urlData['views_max']);
	$urlData['views_allowed'] = translateComma($urlData['views_allowed']);
	$urlData['last_locked']   = generateDateTime($urlData['last_locked'], '2');

	// Is the lock reason empty?
	if (empty($urlData['lock_reason'])) {
		// Fix it to three dashes
		$urlData['lock_reason'] = '---';
	} // END - if

	// Include fields only for action 'edit'
	if ($action == 'edit') {
		// Default is not limited
		$urlData['limited_yes'] = '';
		$urlData['limited_no']  = ' checked="checked"';
		$urlData['limited']     = 'false';

		// Is this URL limited?
		if ($urlData['views_max'] > 0) {
			// Then rewrite form data
			$urlData['limited_yes'] = ' checked="checked"';
			$urlData['limited_no']  = '';
			$urlData['limited']     = 'true';
		} // END - if
	} // END - if

	// Load the form and display it
	LOAD_TEMPLATE(sprintf("member_surfbar_%s_action_form", $action), false, $urlData);

	// All fine by default ... ;-)
	return true;
}

// Execute choosen action
function SURFBAR_MEMBER_EXECUTE_ACTION ($action, $urlData) {
	// By default nothing is executed
	$executed = false;

	// Is limitation "no" and "limit" is > 0?
	if ((isset($urlData[$action]['limited'])) && ($urlData[$action]['limited'] == 'N') && ((isset($urlData[$action]['limit'])) && ($urlData[$action]['limit'] > 0)) || (!isset($urlData[$action]['limit']))) {
		// Set it to unlimited
		$urlData[$action]['limit'] = 0;
	} // END - if

	// Construct function name
	$functionName = sprintf("SURFBAR_MEMBER_EXECUTE_%s_ACTION", strtoupper($action));

	// Is 'userid' set and not 'uid' ?
	if ((!isset($urlData['uid'])) && (isset($urlData['userid']))) {
		// Auto-fix this
		$urlData['uid'] = $urlData['userid'];
	} // END - if

	// Is that function there?
	if (function_exists($functionName)) {
		// Execute the function
		if (call_user_func_array($functionName, array($urlData)) == true) {
			// Update status as well
			$executed = SURFBAR_CHANGE_STATUS($urlData['id'], $urlData['status'], $urlData['new_status'], array($urlData['id'] => $urlData));
		} // END - if
	} else {
		// Not found!
		addFatalMessage(__FUNCTION__, __LINE__, getMessage('MEMBER_SURFBAR_EXECUTE_ACTION_404'), $functionName);
	}

	// Return status
	return $executed;
}
// "Execute edit" function: Update changed data
function SURFBAR_MEMBER_EXECUTE_EDIT_ACTION ($urlData) {
	// Default is nothing done
	$status = false;

	// Translate URLs for testing
	$url1 = COMPILE_CODE($urlData['url']);
	$url2 = COMPILE_CODE($urlData['edit']['url']);

	// Has the URL or limit changed?
	if (true) {
		//if (($urlData['views_allowed'] != $urlData['edit']['limit']) || ($url1 != $url2)) {
		// Run the query
		SQL_QUERY_ESC("UPDATE `{!_MYSQL_PREFIX!}_surfbar_urls` SET url='%s', views_allowed=%s, views_max=%s WHERE `id`=%s AND `status`='%s' LIMIT 1",
		array($urlData['url'], $urlData['edit']['limit'], $urlData['edit']['limit'], $urlData['id'], $urlData['status']), __FUNCTION__, __LINE__);

		// All fine
		$status = true;
	}

	// Return status
	return $status;
}
// "Execute delete" function: Does nothing...
function SURFBAR_MEMBER_EXECUTE_DELETE_ACTION ($urlData) {
	// Nothing special to do (see above function for such "special actions" to perform)
	return true;
}
//
// -----------------------------------------------------------------------------
//                           Self-maintenance functions
// -----------------------------------------------------------------------------
//
// Main function
function SURFBAR_HANDLE_SELF_MAINTENANCE () {
	// Handle URLs which limit has depleted so we can stop them
	SURFBAR_HANDLE_DEPLETED_VIEWS();

	// Handle low-points amounts
	SURFBAR_HANDLE_LOW_POINTS();
}
// Handle URLs which limit has depleted
function SURFBAR_HANDLE_DEPLETED_VIEWS () {
	// Get all URLs
	$urlArray = SURFBAR_GET_URL_DATA('0', 'views_max', 'id', 'ASC', 'id', " AND views_allowed>0 AND `status`='ACTIVE'");

	// Do we have some entries?
	if (count($urlArray) > 0) {
		// Then handle all!
		foreach ($urlArray as $id => $urlData) {
			// Backup data
			$data = $urlData;

			// Rewrite array for next call
			$urlData[$id] = $data;

			// Handle the status
			SURFBAR_CHANGE_STATUS($id, 'ACTIVE', 'DEPLETED', $urlData);
		} // END - foreach
	} // END - if
}

// Alert users which have URLs booked and are low on points amount
function SURFBAR_HANDLE_LOW_POINTS () {
	// Get all userids
	$UIDs = SURFBAR_DETERMINE_DEPLETED_USERIDS(getConfig('surfbar_warn_low_points'));

	// "Walk" through all URLs
	foreach ($UIDs['uid'] as $uid => $dummy) {
		// Is the last notification far enougth away to notify again?
		if ((time() - $UIDs['notified'][$uid]) >= getConfig('surfbar_low_interval')) {
			// Prepare content
			$content = array(
				'uid'      => $uid,
				'low'      => translateComma(getConfig('surfbar_warn_low_points')),
				'points'   => translateComma($UIDs['points'][$uid]),
				'notified' => generateDateTime($UIDs['notified'][$uid]),
				'interval' => createFancyTime(getConfig('surfbar_low_interval'))
			);

			// Notify this user
			SURFBAR_NOTIFY_USER('low_points', $content);

			// Update last notified
			SQL_QUERY_ESC("UPDATE `{!_MYSQL_PREFIX!}_user_data` SET surfbar_low_notified=NOW() WHERE userid=%s LIMIT 1",
			array($uid), __FUNCTION__, __LINE__);
		} // END - if
	} // END - foreach
}

//
// -----------------------------------------------------------------------------
//                               Generic functions
// -----------------------------------------------------------------------------
//

// Looks up by an URL
function SURFBAR_LOOKUP_BY_URL ($url, $uid) {
	// Now lookup that given URL by itself
	$urlArray = SURFBAR_GET_URL_DATA($url, 'url', 'id', 'ASC', 'id', sprintf(" AND `userid`=%s", bigintval($uid)));

	// Was it found?
	return (count($urlArray) > 0);
}

// Load URL data by given search term and column
function SURFBAR_GET_URL_DATA ($searchTerm, $column='id', $order='id', $sort="ASC", $group='id', $add = '') {
	// By default nothing is found
	$GLOBALS['last_url_data'] = array();

	// Is the column an id number?
	if (($column == 'id') || ($column == 'userid')) {
		// Extra secure input
		$searchTerm = bigintval($searchTerm);
	} // END - if

	// If the column is 'id' there can be only one entry
	$limit = '';
	if ($column == 'id') {
		$limit = "LIMIT 1";
	} // END - if

	// Look up the record
	$result = SQL_QUERY_ESC("SELECT id, userid, url, views_total, views_max, views_allowed, status, registered, last_locked, lock_reason, views_max, views_allowed, fixed_reload
FROM `{!_MYSQL_PREFIX!}_surfbar_urls`
WHERE %s='%s'".$add."
ORDER BY %s %s
%s",
	array($column, $searchTerm, $order, $sort, $limit), __FUNCTION__, __LINE__);

	// Is there at least one record?
	if (SQL_NUMROWS($result) > 0) {
		// Then load all!
		while ($dataRow = SQL_FETCHARRAY($result)) {
			// Shall we group these results?
			if ($group == 'id') {
				// Add the row by id as index
				$GLOBALS['last_url_data'][$dataRow['id']] = $dataRow;
			} else {
				// Group entries
				$GLOBALS['last_url_data'][$dataRow[$group]][$dataRow['id']] = $dataRow;
			}
		} // END - while
	} // END - if

	// Free the result
	SQL_FREERESULT($result);

	// Return the result
	return $GLOBALS['last_url_data'];
}

// Registers an URL with the surfbar. You should have called SURFBAR_LOOKUP_BY_URL() first!
function SURFBAR_REGISTER_URL ($url, $uid, $status = 'PENDING', $addMode = 'reg', $extraFields = array()) {
	// Make sure by the user registered URLs are always pending
	if ($addMode == 'reg') {
		$status = 'PENDING';
	} // END - if

	// Prepare content
	$content = merge_array($extraFields, array(
		'url'         => $url,
		'frametester' => FRAMETESTER($url),
		'uid'         => $uid,
		'status'      => $status,
	));

	// Is limit/reload set?
	if (!isset($config['limit'])) {
		$content['limit']  = 0;
	} // END - if
	if (!isset($config['reload'])) {
		$content['reload'] = 0;
	} // END - if

	// Insert the URL into database
	$content['insert_id'] = SURFBAR_INSERT_URL_BY_ARRAY($content);

	// Is this ID valid?
	if ($content['insert_id'] == 0) {
		// INSERT did not insert any data!
		return false;
	} // END - if

	// Translate status and limit
	$content['limit'] = surfbarTranslateLimit($content['limit']);

	// If in reg-mode we notify admin
	if (($addMode == 'reg') || (getConfig('surfbar_notify_admin_unlock') == 'Y')) {
		// Notify admin even when he as unlocked an email
		SURFBAR_NOTIFY_ADMIN("url_{$addMode}", $content);
	} // END - if

	// Send mail to user
	SURFBAR_NOTIFY_USER("url_{$addMode}", $content);

	// Return the insert id
	return $content['insert_id'];
}

// Inserts an url by given data array and return the insert id
function SURFBAR_INSERT_URL_BY_ARRAY ($urlData) {
	// Get userid
	$uid = bigintval($urlData['uid']);

	// Is the id set?
	if (empty($uid)) $uid = 0;

	// Just run the insert query for now
	SQL_QUERY_ESC("INSERT INTO `{!_MYSQL_PREFIX!}_surfbar_urls` (userid,url,status,views_max,views_allowed,fixed_reload) VALUES (%s,'%s','%s',%s,%s,%s)",
	array(
	$uid,
	$urlData['url'],
	$urlData['status'],
	$urlData['limit'],
	$urlData['limit'],
	$urlData['reload']
	), __FUNCTION__, __LINE__
	);

	// Return insert id
	return SQL_INSERTID();
}

// Notify admin(s) with a selected message and content
function SURFBAR_NOTIFY_ADMIN ($messageType, $content) {
	// Prepare template name
	$templateName = sprintf("admin_surfbar_%s", $messageType);

	// Prepare subject
	$subject = getMessage(sprintf("ADMIN_SURFBAR_NOTIFY_%s_SUBJECT",
	strtoupper($messageType)
	));

	// Is the subject line there?
	if ((substr($subject, 0, 1) == '!') && (substr($subject, -1, 1) == '!')) {
		// Set default subject if following eval() wents wrong
		$subject = getMessage('ADMIN_SURFBAR_NOTIFY_DEFAULT_SUBJECT');
	} // END - if

	// Translate some data if present
	if (isset($content['status']))        $content['status']        = surfbarTranslateUserStatus($content['status']);
	if (isset($content['registered']))    $content['registered']    = generateDateTime($content['registered'], '2');
	if (isset($content['last_locked']))   $content['last_locked']   = generateDateTime($content['last_locked'], '2');
	if (isset($content['views_total']))   $content['views_total']   = translateComma($content['views_total']);
	if (isset($content['views_allowed'])) $content['views_allowed'] = translateComma($content['views_allowed']);
	if (isset($content['views_max']))     $content['views_max']     = translateComma($content['views_max']);

	// Send the notification out
	return sendAdminNotification($subject, $templateName, $content, $content['uid']);
}

// Notify the user about the performed action
function SURFBAR_NOTIFY_USER ($messageType, $content) {
	// Skip notification if userid is zero
	if ($content['uid'] == 0) {
		return false;
	} // END - if

	// Prepare template name
	$templateName = sprintf("member_surfbar_%s", $messageType);

	// Prepare subject
	$subject = getMessage(sprintf("MEMBER_SURFBAR_NOTIFY_%s_SUBJECT",
	strtoupper($messageType)
	));

	// Is the subject line there?
	if ((substr($subject, 0, 1) == '!') && (substr($subject, -1, 1) == '!')) {
		// Set default subject if following eval() wents wrong
		$subject = getMessage('MEMBER_SURFBAR_NOTIFY_DEFAULT_SUBJECT');
	} // END - if

	// Translate some data if present
	if (isset($content['status']))        $content['status']        = surfbarTranslateUserStatus($content['status']);
	if (isset($content['registered']))    $content['registered']    = generateDateTime($content['registered'], '2');
	if (isset($content['last_locked']))   $content['last_locked']   = generateDateTime($content['last_locked'], '2');
	if (isset($content['views_total']))   $content['views_total']   = translateComma($content['views_total']);
	if (isset($content['views_allowed'])) $content['views_allowed'] = translateComma($content['views_allowed']);
	if (isset($content['views_max']))     $content['views_max']     = translateComma($content['views_max']);

	// Load template
	$mailText = LOAD_EMAIL_TEMPLATE($templateName, $content, $content['uid']);

	// Send the email
	return sendEmail($content['uid'], $subject, $mailText);
}

// Translates the limit
function surfbarTranslateLimit ($limit) {
	// Is this zero?
	if ($limit == 0) {
		// Unlimited!
		$return = getMessage('MEMBER_SURFBAR_UNLIMITED_VIEWS');
	} else {
		// Translate comma
		$return = translateComma($limit);
	}

	// Return value
	return $return;
}

// Translate the URL status
function surfbarTranslateUserStatus ($status) {
	// Create constant name
	$constantName = sprintf("SURFBAR_URL_STATUS_%s", strtoupper($status));

	// Set default translated status
	$statusTranslated = '!'.$constantName.'!';

	// Is the constant there?
	if (defined($constantName)) {
		// Then get it's value
		$statusTranslated = constant($constantName);
	} // END - if

	// Return result
	return $statusTranslated;
}

// Determine reward
function SURFBAR_DETERMINE_REWARD ($onlyMin=false) {
	// Static values are default
	$reward = getConfig('surfbar_static_reward');

	// Do we have static or dynamic?
	if (getConfig('surfbar_pay_model') == 'DYNAMIC') {
		// "Calculate" dynamic reward
		if ($onlyMin) {
			$reward += SURFBAR_CALCULATE_DYNAMIC_MIN_VALUE();
		} else {
			$reward += SURFBAR_CALCULATE_DYNAMIC_ADD();
		}
	} // END - if

	// Return reward
	return $reward;
}

// Determine costs
function SURFBAR_DETERMINE_COSTS ($onlyMin=false) {
	// Static costs is default
	$costs  = getConfig('surfbar_static_costs');

	// Do we have static or dynamic?
	if (getConfig('surfbar_pay_model') == 'DYNAMIC') {
		// "Calculate" dynamic costs
		if ($onlyMin) {
			$costs += SURFBAR_CALCULATE_DYNAMIC_MIN_VALUE();
		} else {
			$costs += SURFBAR_CALCULATE_DYNAMIC_ADD();
		}
	} // END - if

	// Return costs
	return $costs;
}

// "Calculate" dynamic add
function SURFBAR_CALCULATE_DYNAMIC_ADD () {
	// Get min/max values
	$min = SURFBAR_CALCULATE_DYNAMIC_MIN_VALUE();
	$max = SURFBAR_CALCULATE_DYNAMIC_MAX_VALUE();

	// "Calculate" dynamic part and return it
	return mt_rand($min, $max);
}

// Determine right template name
function SURFBAR_DETERMINE_TEMPLATE_NAME() {
	// Default is the frameset
	$templateName = "surfbar_frameset";

	// Any frame set? ;-)
	if (REQUEST_ISSET_GET('frame')) {
		// Use the frame as a template name part... ;-)
		$templateName = sprintf("surfbar_frame_%s",
		REQUEST_GET('frame')
		);
	} // END - if

	// Return result
	return $templateName;
}

// Check if the "reload lock" of the current user is full, call this function
// before you call SURFBAR_CHECK_RELOAD_LOCK().
function SURFBAR_CHECK_RELOAD_FULL() {
	// Default is full!
	$isFull = true;

	// Cache static reload lock
	$GLOBALS['cache_array']['surfbar']['surf_lock'] = getConfig('surfbar_static_lock');
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "Fixed surf lock is ".getConfig('surfbar_static_lock')."", false);

	// Do we have dynamic model?
	if (getConfig('surfbar_pay_model') == 'DYNAMIC') {
		// "Calculate" dynamic lock
		$GLOBALS['cache_array']['surfbar']['surf_lock'] += SURFBAR_CALCULATE_DYNAMIC_ADD();
	} // END - if

	// Ask the database
	$result = SQL_QUERY_ESC("SELECT COUNT(l.id) AS cnt FROM `{!_MYSQL_PREFIX!}_surfbar_locks` AS l
INNER JOIN `{!_MYSQL_PREFIX!}_surfbar_urls` AS u
ON u.id=l.url_id
WHERE l.userid=%s AND (UNIX_TIMESTAMP() - ".SURFBAR_GET_SURF_LOCK().") < UNIX_TIMESTAMP(l.last_surfed) AND (((UNIX_TIMESTAMP(l.last_surfed) - u.fixed_reload) < 0 AND u.fixed_reload > 0) OR u.fixed_reload = 0)
LIMIT 1",
	array(getUserId()), __FUNCTION__, __LINE__
	);

	// Fetch row
	list($GLOBALS['cache_array']['surfbar']['user_locks']) = SQL_FETCHROW($result);

	// Is it null?
	if (is_null($GLOBALS['cache_array']['surfbar']['user_locks'])) {
		// Then fix it to zero!
		$GLOBALS['cache_array']['surfbar']['user_locks'] = 0;
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Get total URLs
	$total = SURFBAR_GET_TOTAL_URLS();

	// Do we have some URLs in lock? Admins can always surf on own URLs!
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "userLocks=".SURFBAR_GET_USER_LOCKS().",total={$total}", false);
	$isFull = ((SURFBAR_GET_USER_LOCKS() == $total) && ($total > 0));

	// Return result
	return $isFull;
}

// Get total amount of URLs of given status for current user or of ACTIVE URLs by default
function SURFBAR_GET_TOTAL_URLS ($status = 'ACTIVE', $excludeUserId = 0) {
	// Determine depleted user account
	$UIDs = SURFBAR_DETERMINE_DEPLETED_USERIDS();

	// Is the exlude userid set?
	if ($excludeUserId > 0) {
		// Then add it
		$UIDs['uid'][$excludeUserId] = $excludeUserId;
	} // END - if

	// Get amount from database
	$result = SQL_QUERY_ESC("SELECT COUNT(id) AS cnt
FROM `{!_MYSQL_PREFIX!}_surfbar_urls`
WHERE userid NOT IN (".implode(',', $UIDs['uid']).") AND `status`='%s'",
	array($status), __FUNCTION__, __LINE__
	);

	// Fetch row
	list($cnt) = SQL_FETCHROW($result);

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $cnt;
}

// Check wether the user is allowed to book more URLs
function SURFBAR_IF_USER_BOOK_MORE_URLS ($uid=0) {
	// Is this admin and userid is zero or does the user has some URLs left to book?
	return ((($uid == 0) && (IS_ADMIN())) || (SURFBAR_GET_TOTAL_USER_URLS($uid, '', array("REJECTED")) < getConfig('surfbar_max_order')));
}

// Get total amount of URLs of given status for current user
function SURFBAR_GET_TOTAL_USER_URLS ($uid=0, $status = '',$exclude = '') {
	// Is the user 0 and user is logged in?
	if (($uid == 0) && (IS_MEMBER())) {
		// Then use this userid
		$uid = getUserId();
	} elseif ($uid == 0) {
		// Error!
		return (getConfig('surfbar_max_order') + 1);
	}

	// Default is all URLs
	$add = '';

	// Is the status set?
	if (is_array($status)) {
		// Only URLs with these status
		$add = sprintf(" AND status IN('%s')", implode("','", $status));
	} elseif (!empty($status)) {
		// Only URLs with this status
		$add = sprintf(" AND `status`='%s'", $status);
	} elseif (is_array($exclude)) {
		// Exclude URLs with these status
		$add = sprintf(" AND status NOT IN('%s')", implode("','", $exclude));
	} elseif (!empty($exclude)) {
		// Exclude URLs with this status
		$add = sprintf(" AND status != '%s'", $exclude);
	}

	// Get amount from database
	$result = SQL_QUERY_ESC("SELECT COUNT(id) AS cnt
FROM `{!_MYSQL_PREFIX!}_surfbar_urls`
WHERE userid=%s".$add."
LIMIT %s",
	array($uid, getConfig('surfbar_max_order')), __FUNCTION__, __LINE__
	);

	// Fetch row
	list($cnt) = SQL_FETCHROW($result);

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $cnt;
}

// Generate a validation code for the given id number
function SURFBAR_GENERATE_VALIDATION_CODE ($urlId, $salt = '') {
	// @TODO Invalid salt should be refused
	$GLOBALS['cache_array']['surfbar']['salt'] = "INVALID";

	// Get code length from config
	$length = getConfig('code_length');

	// Fix length to 10
	if ($length == 0) $length = 10;

	// Generate a code until the length matches
	$valCode = '';
	while (strlen($valCode) != $length) {
		// Is the salt set?
		if (empty($salt)) {
			// Generate random hashed string
			$GLOBALS['cache_array']['surfbar']['salt'] = sha1(generatePassword(255));
			//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "newSalt=".SURFBAR_GET_SALT()."", false);
		} else {
			// Use this as salt!
			$GLOBALS['cache_array']['surfbar']['salt'] = $salt;
			//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "oldSalt=".SURFBAR_GET_SALT()."", false);
		}

		// ... and now the validation code
		$valCode = generateRandomCode($length, sha1(SURFBAR_GET_SALT().':'.$urlId), getUserId());
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "valCode={$valCode}", false);
	} // END - while

	// Hash it with md5() and salt it with the random string
	$hashedCode = generateHash(md5($valCode), SURFBAR_GET_SALT());

	// Finally encrypt it PGP-like and return it
	$valHashedCode = generatePassString($hashedCode);

	// Return hashed value
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "finalValCode={$valHashedCode}", false);
	return $valHashedCode;
}

// Check validation code
function SURFBAR_CHECK_VALIDATION_CODE ($urlId, $check, $salt) {
	// Secure id number
	$urlId = bigintval($urlId);

	// Now generate the code again
	$code = SURFBAR_GENERATE_VALIDATION_CODE($urlId, $salt);

	// Return result of checking hashes and salts
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, '---'.$code."|".$check.'---', false);
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "+++".$salt."|".SURFBAR_GET_DATA('last_salt')."+++", false);
	return (($code == $check) && ($salt == SURFBAR_GET_DATA('last_salt')));
}

// Lockdown the userid/id combination (reload lock)
function SURFBAR_LOCKDOWN_ID ($urlId) {
	//* DEBUG: */ print "LOCK!");
	///* DEBUG: */ return;
	// Just add it to the database
	SQL_QUERY_ESC("INSERT INTO `{!_MYSQL_PREFIX!}_surfbar_locks` (userid, url_id) VALUES (%s, %s)",
	array(getUserId(), bigintval($urlId)), __FUNCTION__, __LINE__);

	// Remove the salt from database
	SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM `{!_MYSQL_PREFIX!}_surfbar_salts` WHERE url_id=%s AND `userid`=%s LIMIT 1",
	array(bigintval($urlId), getUserId()), __FUNCTION__, __LINE__);
}

// Pay points to the user and remove it from the sender if userid is given else it is a "sponsored surf"
function SURFBAR_PAY_POINTS () {
	// Remove it from the URL owner
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "uid=".SURFBAR_GET_USERID().",costs=".SURFBAR_GET_COSTS()."", false);
	if (SURFBAR_GET_USERID() > 0) {
		SUB_POINTS(sprintf("surfbar_%s", getConfig('surfbar_pay_model')), SURFBAR_GET_USERID(), SURFBAR_GET_COSTS());
	} // END - if

	// Book it to the user
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "uid=".getUserId().",reward=".SURFBAR_GET_REWARD()."", false);
	ADD_POINTS_REFSYSTEM(sprintf("surfbar_%s", getConfig('surfbar_pay_model')), getUserId(), SURFBAR_GET_DATA('reward'));
}

// Updates the statistics of current URL/userid
function SURFBAR_UPDATE_INSERT_STATS_RECORD () {
	// Init add
	$add = '';

	// Get allowed views
	$allowed = SURFBAR_GET_VIEWS_ALLOWED();

	// Do we have a limit?
	if ($allowed > 0) {
		// Then count views_max down!
		$add .= ",views_max=views_max-1";
	} // END - if

	// Update URL stats
	SQL_QUERY_ESC("UPDATE `{!_MYSQL_PREFIX!}_surfbar_urls` SET views_total=views_total+1".$add." WHERE `id`=%s LIMIT 1",
	array(SURFBAR_GET_ID()), __FUNCTION__, __LINE__);

	// Update the stats entry
	SQL_QUERY_ESC("UPDATE `{!_MYSQL_PREFIX!}_surfbar_stats` SET count=count+1 WHERE userid=%s AND url_id=%s LIMIT 1",
	array(getUserId(), SURFBAR_GET_ID()), __FUNCTION__, __LINE__);

	// Was that update okay?
	if (SQL_AFFECTEDROWS() < 1) {
		// No, then insert entry
		SQL_QUERY_ESC("INSERT INTO `{!_MYSQL_PREFIX!}_surfbar_stats` (userid,url_id,count) VALUES (%s,%s,1)",
		array(getUserId(), SURFBAR_GET_ID()), __FUNCTION__, __LINE__);
	} // END - if

	// Update total/daily/weekly/monthly counter
	incrementConfigEntry('surfbar_total_counter');
	incrementConfigEntry('surfbar_daily_counter');
	incrementConfigEntry('surfbar_weekly_counter');
	incrementConfigEntry('surfbar_monthly_counter');

	// Update config as well
	updateConfiguration(array('surfbar_total_counter', 'surfbar_daily_counter', 'surfbar_weekly_counter', 'surfbar_monthly_counter'), array(1,1,1,1), '+');
}

// Update the salt for validation and statistics
function SURFBAR_UPDATE_SALT_STATS () {
	// Update statistics record
	SURFBAR_UPDATE_INSERT_STATS_RECORD();

	// Simply store the salt from cache away in database...
	SQL_QUERY_ESC("UPDATE `{!_MYSQL_PREFIX!}_surfbar_salts` SET last_salt='%s' WHERE url_id=%s AND `userid`=%s LIMIT 1",
	array(SURFBAR_GET_SALT(), SURFBAR_GET_ID(), getUserId()), __FUNCTION__, __LINE__);

	// Debug message
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "salt=".SURFBAR_GET_SALT().",id=".SURFBAR_GET_ID().",uid=".getUserId()."", false);

	// Was that okay?
	if (SQL_AFFECTEDROWS() < 1) {
		// Insert missing entry!
		SQL_QUERY_ESC("INSERT INTO `{!_MYSQL_PREFIX!}_surfbar_salts` (url_id,userid,last_salt) VALUES (%s, %s, '%s')",
		array(SURFBAR_GET_ID(), getUserId(), SURFBAR_GET_SALT()), __FUNCTION__, __LINE__);
	} // END - if

	// Debug message
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "affectedRows=".SQL_AFFECTEDROWS()."", false);

	// Return if the update was okay
	return (SQL_AFFECTEDROWS() == 1);
}

// Check if the reload lock is active for given id
function SURFBAR_CHECK_RELOAD_LOCK ($urlId) {
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "id={$urlId}", false);
	// Ask the database
	$result = SQL_QUERY_ESC("SELECT COUNT(id) AS cnt
FROM `{!_MYSQL_PREFIX!}_surfbar_locks`
WHERE userid=%s AND url_id=%s AND (UNIX_TIMESTAMP() - ".SURFBAR_GET_SURF_LOCK().") < UNIX_TIMESTAMP(last_surfed)
ORDER BY last_surfed ASC
LIMIT 1",
	array(getUserId(), bigintval($urlId)), __FUNCTION__, __LINE__
	);

	// Fetch counter
	list($cnt) = SQL_FETCHROW($result);

	// Free result
	SQL_FREERESULT($result);

	// Return check
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "cnt={$cnt},".SURFBAR_GET_SURF_LOCK()."", false);
	return ($cnt == 1);
}

// Determine which user hash no more points left
function SURFBAR_DETERMINE_DEPLETED_USERIDS ($limit=0) {
	// Init array
	$UIDs = array(
		'uid'      => array(),
		'points'   => array(),
		'notified' => array(),
	);

	// Do we have a current user id?
	if ((IS_MEMBER()) && ($limit == 0)) {
		// Then add this as well
		$UIDs['uid'][getUserId()]      = getUserId();
		$UIDs['points'][getUserId()]   = GET_TOTAL_DATA(getUserId(), "user_points", "points") - GET_TOTAL_DATA(getUserId(), "user_data", "used_points");
		$UIDs['notified'][getUserId()] = 0;

		// Get all userid except logged in one
		$result = SQL_QUERY_ESC("SELECT u.userid, UNIX_TIMESTAMP(d.surfbar_low_notified) AS notified
FROM `{!_MYSQL_PREFIX!}_surfbar_urls` AS u
INNER JOIN `{!_MYSQL_PREFIX!}_user_data` AS d
ON u.userid=d.userid
WHERE u.userid NOT IN (%s,0) AND u.`status`='ACTIVE'
GROUP BY u.userid
ORDER BY u.userid ASC",
		array(getUserId()), __FUNCTION__, __LINE__);
	} else {
		// Get all userid
		$result = SQL_QUERY("SELECT u.userid, UNIX_TIMESTAMP(d.surfbar_low_notified) AS notified
FROM `{!_MYSQL_PREFIX!}_surfbar_urls` AS u
INNER JOIN `{!_MYSQL_PREFIX!}_user_data` AS d
ON u.userid=d.userid
WHERE u.`status`='ACTIVE'
GROUP BY u.userid
ORDER BY u.userid ASC", __FUNCTION__, __LINE__);
	}

	// Load all userid
	while ($content = SQL_FETCHARRAY($result)) {
		// Get total points
		$points = GET_TOTAL_DATA($content['userid'], "user_points", "points") - GET_TOTAL_DATA($content['userid'], "user_data", "used_points");
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "uid={$content['userid']},points={$points}", false);

		// Shall we add this to ignore?
		if ($points <= $limit) {
			// Ignore this one!
			//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "uid={$content['userid']} has depleted points amount!", false);
			$UIDs['uid'][$content['userid']]      = $content['userid'];
			$UIDs['points'][$content['userid']]   = $points;
			$UIDs['notified'][$content['userid']] = $content['notified'];
		} // END - if
	} // END - while

	// Free result
	SQL_FREERESULT($result);

	// Debug message
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "UIDs::count=".count($UIDs)." (with own userid=".getUserId().')', false);

	// Return result
	return $UIDs;
}

// Determine how many users are Online in surfbar
function SURFBAR_DETERMINE_TOTAL_ONLINE () {
	// Count all users in surfbar modue and return the value
	$result = SQL_QUERY_ESC("SELECT id
FROM `{!_MYSQL_PREFIX!}_surfbar_stats`
WHERE (UNIX_TIMESTAMP() - UNIX_TIMESTAMP(last_online)) <= %s
GROUP BY userid",
	array(getConfig('online_timeout')), __FUNCTION__, __LINE__);

	// Fetch count
	$cnt = SQL_NUMROWS($result);

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $cnt;
}

// Determine waiting time for one URL
function SURFBAR_DETERMINE_WAIT_TIME () {
	// Get fixed reload lock
	$fixed = SURFBAR_GET_FIXED_RELOAD();

	// Is the fixed reload time set?
	if ($fixed > 0) {
		// Return it
		return $fixed;
	} // END - if

	// Static time is default
	$time = getConfig('surfbar_static_time');

	// Which payment model do we have?
	if (getConfig('surfbar_pay_model') == 'DYNAMIC') {
		// "Calculate" dynamic time
		$time += SURFBAR_CALCULATE_DYNAMIC_ADD();
	} // END - if

	// Return value
	return $time;
}

// Changes the status of an URL from given to other
function SURFBAR_CHANGE_STATUS ($urlId, $prevStatus, $newStatus, $data=array()) {
	// Make new status always lower-case
	$newStatus = strtolower($newStatus);

	// Get URL data for status comparison if missing
	if ((!is_array($data)) || (count($data) == 0)) {
		// Fetch missing URL data
		$data = SURFBAR_GET_URL_DATA($urlId);
	} // END - if

	// Is the new status set?
	if ((!is_string($newStatus)) || (empty($newStatus))) {
		// Abort here, but fine!
		return true;
	} // END - if

	// Is the status like prevStatus is saying?
	if ($data[$urlId]['status'] != $prevStatus) {
		// No, then abort here
		return false;
	} // END - if


	// Update the status now
	// ---------- Comment out for debugging/developing member actions! ---------
	//SQL_QUERY_ESC("UPDATE `{!_MYSQL_PREFIX!}_surfbar_urls` SET `status`='%s' WHERE `id`=%s LIMIT 1",
	//	array($newStatus, bigintval($urlId)), __FUNCTION__, __LINE__);
	// ---------- Comment out for debugging/developing member actions! ---------

	// Was that fine?
	// ---------- Comment out for debugging/developing member actions! ---------
	//if (SQL_AFFECTEDROWS() != 1) {
	//	// No, something went wrong
	//	return false;
	//} // END - if
	// ---------- Comment out for debugging/developing member actions! ---------

	// Prepare content for notification routines
	$data[$urlId]['uid']         = $data[$urlId]['userid'];
	$data[$urlId]['frametester'] = FRAMETESTER($data[$urlId]['url']);
	$data[$urlId]['reward']      = translateComma(getConfig('surfbar_static_reward'));
	$data[$urlId]['costs']       = translateComma(getConfig('surfbar_static_costs'));

	// Do some dirty fixing here:
	if (($data[$urlId]['status'] == 'STOPPED') && ($newStatus == 'pending')) {
		// Fix for template change
		$newStatus = 'continued';
	} // END - if

	// Send admin notification
	SURFBAR_NOTIFY_ADMIN("url_{$data[$urlId]['status']}_{$newStatus}", $data[$urlId]);

	// Send user notification
	SURFBAR_NOTIFY_USER("url_{$data[$urlId]['status']}_{$newStatus}", $data[$urlId]);

	// All done!
	return true;
}

// Calculate minimum value for dynamic payment model
function SURFBAR_CALCULATE_DYNAMIC_MIN_VALUE () {
	// Addon is zero by default
	$addon = 0;

	// Percentage part
	$percent = abs(log(getConfig('surfbar_dynamic_percent') / 100 + 1));

	// Get total users
	$totalUsers = GET_TOTAL_DATA('CONFIRMED', 'user_data', 'userid', 'status', true);

	// Get online users
	$onlineUsers = SURFBAR_DETERMINE_TOTAL_ONLINE();

	// Calculate addon
	$addon += abs(log($onlineUsers / $totalUsers + 1) * $percent * $totalUsers);

	// Get total URLs
	$totalUrls = SURFBAR_GET_TOTAL_URLS('ACTIVE', '0');

	// Get user's total URLs
	$userUrls = SURFBAR_GET_TOTAL_USER_URLS(0, 'ACTIVE');

	// Calculate addon
	if ($totalUrls > 0) {
		$addon += abs(log($userUrls / $totalUrls + 1) * $percent * $totalUrls);
	} else {
		$addon += abs(log($userUrls / 1 + 1) * $percent * $totalUrls);
	}

	// Return addon
	return $addon;
}

// Calculate maximum value for dynamic payment model
function SURFBAR_CALCULATE_DYNAMIC_MAX_VALUE () {
	// Addon is zero by default
	$addon = 0;

	// Maximum value
	$max = log(2);

	// Percentage part
	$percent = abs(log(getConfig('surfbar_dynamic_percent') / 100 + 1));

	// Get total users
	$totalUsers = GET_TOTAL_DATA('CONFIRMED', 'user_data', 'userid', 'status', true);

	// Calculate addon
	$addon += abs($max * $percent * $totalUsers);

	// Get total URLs
	$totalUrls = SURFBAR_GET_TOTAL_URLS('ACTIVE', '0');

	// Calculate addon
	$addon += abs($max * $percent * $totalUrls);

	// Return addon
	return $addon;
}

// Calculate dynamic lock
function SURFBAR_CALCULATE_DYNAMIC_LOCK () {
	// Default lock is 30 seconds
	$addon = 30;

	// Get online users
	$onlineUsers = SURFBAR_DETERMINE_TOTAL_ONLINE();

	// Calculate lock
	$addon = abs(log($onlineUsers / $addon + 1));

	// Return value
	return $addon;
}

// "Getter" for lock ids array
function SURFBAR_GET_LOCK_IDS () {
	// Prepare some arrays
	$IDs = array();
	$USE = array();
	$ignored = array();

	// Get all id from locks within the timestamp
	$result = SQL_QUERY_ESC("SELECT id, url_id, UNIX_TIMESTAMP(last_surfed) AS last
FROM
	{!_MYSQL_PREFIX!}_surfbar_locks
WHERE
	userid=%s
ORDER BY
	id ASC", array(getUserId()),
	__FUNCTION__, __LINE__);

	// Load all entries
	while ($content = SQL_FETCHARRAY($result)) {
		// Debug message
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "next - lid={$content['id']},url={$content['url_id']},rest=".(time() - $last).'/'.SURFBAR_GET_SURF_LOCK()."", false);

		// Skip entries that are too old
		if (($last > (time() - SURFBAR_GET_SURF_LOCK())) && (!in_array($content['url_id'], $ignored))) {
			// Debug message
			//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "okay - lid={$content['id']},url={$content['url_id']},last={$last}", false);

			// Add only if missing or bigger
			if ((!isset($IDs[$content['url_id']])) || ($IDs[$content['url_id']] > $last)) {
				// Debug message
				//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "ADD - lid={$content['id']},url={$content['url_id']},last={$last}", false);

				// Add this ID
				$IDs[$content['url_id']] = $last;
				$USE[$content['url_id']] = $content['id'];
			} // END - if
		} else {
			// Debug message
			//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "ignore - lid={$content['id']},url={$content['url_id']},last={$last}", false);

			// Ignore these old entries!
			$ignored[] = $content['url_id'];
			unset($IDs[$content['url_id']]);
			unset($USE[$content['url_id']]);
		}
	} // END - while

	// Free result
	SQL_FREERESULT($result);

	// Return array
	return $USE;
}

// "Getter" for maximum random number
function SURFBAR_GET_MAX_RANDOM ($UIDs, $add) {
	// Count max availabe entries
	$result = SQL_QUERY("SELECT sbu.id AS cnt
FROM `{!_MYSQL_PREFIX!}_surfbar_urls` AS sbu
LEFT JOIN `{!_MYSQL_PREFIX!}_surfbar_salts` AS sbs
ON sbu.id=sbs.url_id
LEFT JOIN `{!_MYSQL_PREFIX!}_surfbar_locks` AS l
ON sbu.id=l.url_id
WHERE sbu.userid NOT IN (".implode(',', $UIDs).") AND (sbu.views_allowed=0 OR (sbu.views_allowed > 0 AND sbu.views_max > 0)) AND sbu.`status`='ACTIVE'".$add."
GROUP BY sbu.id", __FUNCTION__, __LINE__);

	// Log last query
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "lastQuery=".getConfig('db_last_query')."|numRows=".SQL_NUMROWS($result)."|Affected=".SQL_AFFECTEDROWS()."", false);

	// Fetch max rand
	$maxRand = SQL_NUMROWS($result);

	// Free result
	SQL_FREERESULT($result);

	// Return value
	return $maxRand;
}

// Load all URLs of the current user and return it as an array
function SURFBAR_GET_USER_URLS () {
	// Init array
	$URLs = array();

	// Begin the query
	$result = SQL_QUERY_ESC("SELECT u.id, u.userid, u.url, u.views_total, u.views_max, u.views_allowed, u.status, UNIX_TIMESTAMP(u.registered) AS registered, UNIX_TIMESTAMP(u.last_locked) AS last_locked, u.lock_reason AS lock_reason
FROM `{!_MYSQL_PREFIX!}_surfbar_urls` AS u
WHERE u.userid=%s AND u.status != 'DELETED'
ORDER BY u.id ASC",
	array(getUserId()), __FUNCTION__, __LINE__);

	// Are there entries?
	if (SQL_NUMROWS($result) > 0) {
		// Load all rows
		while ($row = SQL_FETCHARRAY($result)) {
			// Add the row
			$URLs[$row['id']] = $row;
		} // END - while
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return the array
	return $URLs;
}

// "Getter" for member action array for given status
function SURFBAR_getModeAction_ARRAY ($status) {
	// Init array
	$returnArray = array();

	// Get all assigned actions
	$result = SQL_QUERY_ESC("SELECT action FROM `{!_MYSQL_PREFIX!}_surfbar_actions` WHERE `status`='%s' ORDER BY `id` ASC",
	array($status), __FUNCTION__, __LINE__);

	// Some entries there?
	if (SQL_NUMROWS($result) > 0) {
		// Load all actions
		// @TODO This can be somehow rewritten
		while ($content = SQL_FETCHARRAY($result)) {
			$returnArray[] = $content['action'];
		} // END - if
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $returnArray;
}

// Reload to configured stop page
function SURFBAR_RELOAD_TO_STOP_PAGE ($page="stop") {
	// Internal or external?
	if ((getConfig('surfbar_pause_mode') == 'INTERNAL') || (getConfig('surfbar_pause_url') == '')) {
		// Reload to internal page
		redirectToUrl('surfbar.php?frame=' . $page);
	} else {
		// Reload to external page
		redirectToConfiguredUrl('surfbar_pause_url');
	}
}

// Determine next id for surfbar or get data for given id, always call this before you call other
// getters below this function!!!
function SURFBAR_DETERMINE_NEXT_ID ($urlId = 0) {
	// Default is no id and no random number
	$nextId = 0;
	$randNum = 0;

	// Is the ID set?
	if ($urlId == 0) {
		// Get array with lock ids
		$USE = SURFBAR_GET_LOCK_IDS();

		// Shall we add some URL ids to ignore?
		$add = '';
		if (count($USE) > 0) {
			// Ignore some!
			$add = " AND sbu.id NOT IN (";
			foreach ($USE as $url_id => $lid) {
				// Add URL id
				$add .= $url_id.',';
			} // END - foreach

			// Add closing bracket
			$add = substr($add, 0, -1) . ')';
		} // END - if

		// Determine depleted user account
		$UIDs = SURFBAR_DETERMINE_DEPLETED_USERIDS();

		// Get maximum randomness factor
		$maxRand = SURFBAR_GET_MAX_RANDOM($UIDs['uid'], $add);

		// If more than one URL can be called generate the random number!
		if ($maxRand > 1) {
			// Generate random number
			$randNum = mt_rand(0, ($maxRand - 1));
		} // END - if

		// And query the database
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "randNum={$randNum},maxRand={$maxRand},surfLock=".SURFBAR_GET_SURF_LOCK()."", false);
		$result = SQL_QUERY_ESC("SELECT sbu.id, sbu.userid, sbu.url, sbs.last_salt, sbu.views_total, sbu.views_max, sbu.views_allowed, UNIX_TIMESTAMP(l.last_surfed) AS last_surfed, sbu.fixed_reload
FROM `{!_MYSQL_PREFIX!}_surfbar_urls` AS sbu
LEFT JOIN `{!_MYSQL_PREFIX!}_surfbar_salts` AS sbs
ON sbu.id=sbs.url_id
LEFT JOIN `{!_MYSQL_PREFIX!}_surfbar_locks` AS l
ON sbu.id=l.url_id
WHERE sbu.userid NOT IN (".implode(',', $UIDs['uid']).") AND sbu.`status`='ACTIVE' AND (sbu.views_allowed=0 OR (sbu.views_allowed > 0 AND sbu.views_max > 0))".$add."
GROUP BY sbu.id
ORDER BY l.last_surfed ASC, sbu.id ASC
LIMIT %s,1",
		array($randNum), __FUNCTION__, __LINE__
		);
	} else {
		// Get data from specified id number
		$result = SQL_QUERY_ESC("SELECT sbu.id, sbu.userid, sbu.url, sbs.last_salt, sbu.views_total, sbu.views_max, sbu.views_allowed, UNIX_TIMESTAMP(l.last_surfed) AS last_surfed, sbu.fixed_reload
FROM `{!_MYSQL_PREFIX!}_surfbar_urls` AS sbu
LEFT JOIN `{!_MYSQL_PREFIX!}_surfbar_salts` AS sbs
ON sbu.id=sbs.url_id
LEFT JOIN `{!_MYSQL_PREFIX!}_surfbar_locks` AS l
ON sbu.id=l.url_id
WHERE sbu.userid != %s AND sbu.`status`='ACTIVE' AND sbu.id=%s AND (sbu.views_allowed=0 OR (sbu.views_allowed > 0 AND sbu.views_max > 0))
LIMIT 1",
		array(getUserId(), bigintval($urlId)), __FUNCTION__, __LINE__
		);
	}

	// Is there an id number?
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "lastQuery=".getConfig('db_last_query')."|numRows=".SQL_NUMROWS($result)."|Affected=".SQL_AFFECTEDROWS()."", false);
	if (SQL_NUMROWS($result) == 1) {
		// Load/cache data
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "count(".count($GLOBALS['cache_array']['surfbar']).") - BEFORE", false);
		$GLOBALS['cache_array']['surfbar'] = merge_array($GLOBALS['cache_array']['surfbar'], SQL_FETCHARRAY($result));
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "count(".count($GLOBALS['cache_array']['surfbar']).") - AFTER", false);

		// Determine waiting time
		$GLOBALS['cache_array']['surfbar']['time'] = SURFBAR_DETERMINE_WAIT_TIME();

		// Is the last salt there?
		if (is_null($GLOBALS['cache_array']['surfbar']['last_salt'])) {
			// Then repair it wit the static!
			//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "last_salt - FIXED!", false);
			$GLOBALS['cache_array']['surfbar']['last_salt'] = '';
		} // END - if

		// Fix missing last_surfed
		if ((!isset($GLOBALS['cache_array']['surfbar']['last_surfed'])) || (is_null($GLOBALS['cache_array']['surfbar']['last_surfed']))) {
			// Fix it here
			//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "last_surfed - FIXED!", false);
			$GLOBALS['cache_array']['surfbar']['last_surfed'] = 0;
		} // END - if

		// Get base/fixed reward and costs
		$GLOBALS['cache_array']['surfbar']['reward'] = SURFBAR_DETERMINE_REWARD();
		$GLOBALS['cache_array']['surfbar']['costs']  = SURFBAR_DETERMINE_COSTS();
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "BASE/STATIC - reward=".SURFBAR_GET_REWARD()."|costs=".SURFBAR_GET_COSTS()."", false);

		// Only in dynamic model add the dynamic bonus!
		if (getConfig('surfbar_pay_model') == 'DYNAMIC') {
			// Calculate dynamic reward/costs and add it
			$GLOBALS['cache_array']['surfbar']['reward'] += SURFBAR_CALCULATE_DYNAMIC_ADD();
			$GLOBALS['cache_array']['surfbar']['costs']  += SURFBAR_CALCULATE_DYNAMIC_ADD();
			//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "DYNAMIC+ - reward=".SURFBAR_GET_REWARD()."|costs=".SURFBAR_GET_COSTS()."", false);
		} // END - if

		// Now get the id
		$nextId = SURFBAR_GET_ID();
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return result
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "nextId={$nextId}", false);
	return $nextId;
}

// -----------------------------------------------------------------------------
// PLEASE DO NOT ADD ANY OTHER FUNCTIONS BELOW THIS LINE IF THEY DON'T "WRAP"
// THE $GLOBALS['cache_array']['surfbar'] ARRAY!
// -----------------------------------------------------------------------------
// Private getter for data elements
function SURFBAR_GET_DATA ($element) {
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "element={$element}", false);

	// Default is null
	$data = null;

	// Is the entry there?
	if (isset($GLOBALS['cache_array']['surfbar'][$element])) {
		// Then take it
		$data = $GLOBALS['cache_array']['surfbar'][$element];
	} else { // END - if
		print("<pre>");
		print_r($GLOBALS['cache_array']['surfbar']);
		print("</pre>");
		debug_report_bug();
	}

	// Return result
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "element[$element]={$data}", false);
	return $data;
}

// Getter for reward from cache
function SURFBAR_GET_REWARD () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('reward');
}

// Getter for costs from cache
function SURFBAR_GET_COSTS () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('costs');
}

// Getter for URL from cache
function SURFBAR_GET_URL () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('url');
}

// Getter for salt from cache
function SURFBAR_GET_SALT () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('salt');
}

// Getter for id from cache
function SURFBAR_GET_ID () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('id');
}

// Getter for userid from cache
function SURFBAR_GET_USERID () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('userid');
}

// Getter for user reload locks
function SURFBAR_GET_USER_LOCKS () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('user_locks');
}

// Getter for reload time
function SURFBAR_GET_RELOAD_TIME () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('time');
}

// Getter for allowed views
function SURFBAR_GET_VIEWS_ALLOWED () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('views_allowed');
}

// Getter for fixed reload
function SURFBAR_GET_FIXED_RELOAD () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('fixed_reload');
}

// Getter for surf lock
function SURFBAR_GET_SURF_LOCK () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('surf_lock');
}

// [EOF]
?>
